<?php
/* Check that files that are supposed to be identical are indeed identical

	Usage:

		php verify_same.php [-c] [-l en,fr] [-m u1,eu]

	Parameters:

	-c: Generate copy instructions instead of diff instructions. Optional
	-l language(s): comma-separated list of languages to evaluate. Default: all
	-m map(s): comma-separated list of maps to evaluate. Default: all
*/

// Change the following two tables depending on your needs
// The first table lists the files that should be identical to the 'us' map.
// The second table is an exception list which superseeds the first table.


$opts = getopt('cvl:m:');
$gCopy = isset($opts['c']);
$gVerbose = isset($opts['v']);

if ($opts['l'])
	$languages = explode(',', $opts['l']);
else
	$languages = array('en', 'fr', 'de', 'ja', 'zh');
if ($opts['m'])
	$maps = explode(',', $opts['m']);
else
	$maps = array('eu', 'u1', 'u2', 'u3', 'ch', 'as'); // Note that the us map is not listed, because it's the reference version

$same_ones = array(
  //array('file'=>'index.html'),  // index are ALL different
  //array('file'=>'WhatsNew.html'),  // What's new are ALL different
  array('file'=>'Goal.html', 'prev'=>true),
  array('file'=>'GameTurn.html'),
  array('file'=>'EndGame.html'),
  array('file'=>'Tickets.html', 'next'=>true),
  array('file'=>'TrainCards.html', 'prev'=>true),
  array('file'=>'Locomotives.html'),
  array('file'=>'Trains.html'),
  array('file'=>'Routes.html', 'next'=>true),
  array('file'=>'Cities.html', 'prev'=>true, 'next'=>true),
  array('file'=>'Scoring.html'),
  array('file'=>'RoutesScoring.html'),
  array('file'=>'BonusCardsScoring.html'),
  //array('file'=>'GameStats.html'),  // Game stats are ALL different
  array('file'=>'Extras.html'),
  array('file'=>'Karma.html'),
  array('file'=>'ELO.html'),
  array('file'=>'LocalPlay.html'),
  array('file'=>'Achievements.html'),

);

// Use the 'map', 'language' and 'file' keys to list exceptions
$exceptions = array(
  array('map'=>'u1', 'file'=>'Goal.html'),
  array('map'=>'u2', 'file'=>'Goal.html'),
  array('map'=>'as', 'file'=>'Goal.html'),
  array('map'=>'eu', 'file'=>'GameTurn.html'),
  array('map'=>'ch', 'file'=>'Trains.html'),
  array('map'=>'eu', 'file'=>'Locomotives.html'), // Locomotives are also vital to claiming <a href="Ferries.html">Ferry</a> routes
  array('map'=>'ch', 'file'=>'Locomotives.html'), // If a Locomotive is picked face-up during a card draw, it must be the only card picked that turn.
  array('map'=>'as', 'file'=>'Locomotives.html'), // Locomotives are also vital to claiming <a href="Ferries.html">Ferry</a> routes
  array('map'=>'ch', 'file'=>'Routes.html'), // In 2 player games once a player claims a Double-Routes
  array('map'=>'eu', 'file'=>'Scoring.html'),   // <li>for any unbuilt <a href="Stations.html">Stations</a></li>
  array('map'=>'as', 'file'=>'Scoring.html'),   // You score points each time you claim a <a href="RoutesScoring.html">Route</a>; discard Trains
  array('map'=>'eu', 'file'=>'RoutesScoring.html'),	// routesScoring_eu.png
  array('map'=>'u1', 'file'=>'BonusCardsScoring.html'),
  array('map'=>'u2', 'file'=>'BonusCardsScoring.html'),
  array('map'=>'u3', 'file'=>'BonusCardsScoring.html'),
  array('map'=>'as', 'file'=>'BonusCardsScoring.html'),
);

$path = dirname(__FILE__) . '/..';

function compare_files($file1, $file2, $diff_prev, $diff_next)
{
  global $gCopy;
  global $gVerbose;
  $content1 = file_get_contents($file1);
  $content2 = @file_get_contents($file2);
  if ($content2 !== false)
  {
    if ($diff_prev)
    {
      $content1 = preg_replace('!<a id="prev".*?<\/a>!', '', $content1);
      $content2 = preg_replace('!<a id="prev".*?<\/a>!', '', $content2);
    }
    if ($diff_next)
    {
      $content1 = preg_replace('!<a id="next".*?<\/a>!', '', $content1);
      $content2 = preg_replace('!<a id="next".*?<\/a>!', '', $content2);
    }
    if ($content1 != $content2)
    {
	  if ($gCopy)
	    echo "cp -v $file1 $file2\n";
	  else
        echo "# Different from US map: $file2 -- diff $file1 $file2\n";
    }
  }
}

function is_exception($map, $language, $file)
{
  global $exceptions;
  foreach($exceptions as $exception)
  {
    $nb_to_check = count($exception);
    $found = 0;
    if ($exception['map'] && $exception['map']==$map) $found++;
    if ($exception['language'] && $exception['language']==$language) $found++;
    if ($exception['file'] && $exception['file']==$file) $found++;
    if ($nb_to_check && $nb_to_check==$found) return true;
  }
  return false;
}


foreach($languages as $language)
{
	if ($gVerbose) echo "# Language: $language\n";
  foreach($same_ones as $candidate)
  {
    $file1 = $path . '/' . 'us' . '/'. $language. '/' . $candidate['file'];  // The US version is always the reference file
		if ($gVerbose) echo "## File: {$candidate['file']}\n";
    foreach($maps as $map)
    {
			if ($gVerbose) echo "### Map: {$map}\n";
      $file2 = $path . '/' . $map . '/'. $language. '/' . $candidate['file'];
      if (!file_exists($file2))
      {
        echo "# Missing file: $file2\n";
      }
      elseif (!is_exception($map, $language, $candidate['file']))
      {
        compare_files($file1, $file2, $candidate['prev'], $candidate['next']);
      }
    }
  }
}
